/*******************************************************************************
   File:
      CLOCK.C

   Purpose:
      Game Timer for the Pilot

   $Revision: 1.11 $
   $Header: D:\devel\pilot\CodeWarrior\GameTimer\Src/clock.c,v 1.11 2001/01/24 03:31:00 eshayne Exp $
   $Log: clock.c,v $
 * Revision 1.11  2001/01/24  03:31:00  eshayne
 * fixed problem with fischer time hitting at end of turn
 *
 * Revision 1.10  2001/01/22  04:24:29  eshayne
 * removed deprecated original preferences screen
 *
 * Revision 1.9  2001/01/22  04:17:05  eshayne
 * added miscellaneous settings screen
 *
 * Revision 1.8  2001/01/22  03:57:32  eshayne
 * added adjustment preferences screen
 *
 * Revision 1.7  2001/01/22  03:10:03  eshayne
 * added new preferences screen for startup times
 *
 * Revision 1.6  2001/01/22  01:50:12  eshayne
 * added new preferences: handicap, fischer adjustment @ end of turn and reset after each turn
 *
 * Revision 1.5  2000/09/10  23:22:26  eshayne
 * changed name to GameTimer
 *
 * Revision 1.4  2000/09/10  23:10:54  eshayne
 * renamed to GameTimer and adjusted all screens
 * and filenames
 *
 * Revision 1.3  2000/09/10  23:03:51  eshayne
 * renamed to Game Timer
 * default Fischer Time to 0 (disabled)
 * option to disable auto-off
 *
 * Revision 1.2  2000/09/09  23:56:47  eshayne
 * renamed resource file header
 *
 * Revision 1.1  2000/09/09  23:23:54  eshayne
 * Initial revision
 *
 * Revision 1.36  2000/05/07  18:39:11  eshayne
 * removed ^M's
 *
 * Revision 1.35  1997/09/14  00:09:51  eshayne
 * if hardware buttons are enabled, either button can start the initial
 * countdown
 *
 * Revision 1.34  1997/09/13  22:49:21  eshayne
 * hide software buttons if hardware buttons are enabled
 *
 * Revision 1.33  1997/09/13  22:32:45  eshayne
 * added confirmation if enable hardware buttons
 *
 * Revision 1.32  1997/09/13  22:24:05  eshayne
 * added preferences option to enable or disable hardware buttons
 *
 * Revision 1.31  1997/09/13  21:28:23  eshayne
 * added hardware buttons 1 & 4 to be substitutable for the
 * "switch" buttons
 *
 * Revision 1.30  1997/09/13  21:10:57  eshayne
 * made two "switch" buttons, one for each player
 *
 * Revision 1.29  1997/09/08  00:55:42  eshayne
 * merged common functionality from inital setup and 
  into two
 * helper functions, SetInitialValues() and DrawInitialFields() for
 * before and after the form is drawn, respectively.
 *
 * Revision 1.28  1997/09/07  19:11:25  eshayne
 * added GNU Public License to source files
 *
 * Revision 1.27  1997/09/07  18:33:40  eshayne
 * replaced global preference vars with one instance of preference struct
 *
 * Revision 1.26  1997/09/07  18:09:34  eshayne
 * linked creator-id in code to creator-id in Makefile
 *
 * Revision 1.25  1997/09/07  18:02:26  eshayne
 * made preferences persistent
 *
 * Revision 1.24  1997/09/01  23:41:59  eshayne
 * added optional pause button
 *
 * Revision 1.23  1997/09/01  20:58:08  eshayne
 * made switching routine more accurate (I think) by capturing amount
 * of time left before next decrement before switching over - so next
 * time we can only wait that long before decrementing again. I'm not
 * convinced it works, but it's pretty much impossible to test...
 *
 * Revision 1.22  1997/09/01  19:34:02  eshayne
 * made button switch label between "Start" and "Switch," appropriately
 *
 * Revision 1.21  1997/09/01  19:09:39  eshayne
 * added popup/beep when time is out for one player
 *
 * Revision 1.20  1997/09/01  18:55:00  eshayne
 * added validation for preferences screen
 *
 * Revision 1.19  1997/09/01  18:20:44  eshayne
 * preset preferences screen with current values
 *
 * Revision 1.18  1997/09/01  17:24:36  eshayne
 * removed "Fischer Time?" checkbox
 * made preferences actually count if user hits "OK"
 *
 * Revision 1.17  1997/09/01  16:37:26  eshayne
 * added confirmation dialog on reset
 *
 * Revision 1.16  1997/09/01  16:31:20  eshayne
 * added Reset option to menu
 *
 * Revision 1.15  1997/09/01  16:08:30  eshayne
 * added Fischer time
 *
 * Revision 1.14  1997/09/01  14:43:52  eshayne
 * moved all TimeCounter-related stuff to counter.h/.c
 *
 * Revision 1.13  1997/09/01  14:31:12  eshayne
 * moved global variable definitions to clockGlobals.c
 *
 * Revision 1.12  1997/09/01  14:21:44  eshayne
 * form now initially shows initial counter values before first "switch"
 *
 * Revision 1.11  1997/08/31  21:50:51  eshayne
 * added Settings menu option
 * added Preferences form to real app
 * Preferences form settings still ignored, though
 *
 * Revision 1.10  1997/08/31  19:22:15  eshayne
 * added leading 0's
 *
 * Revision 1.9  1997/08/31  19:12:38  eshayne
 * added "About" dialog
 * added menu
 * added resources for preferences dialog, but no hooks yet
 *
 * Revision 1.8  1997/08/31  02:34:55  eshayne
 * added comments
 *
 * Revision 1.7  1997/08/31  02:25:51  eshayne
 * extern-ized globals
 *
 * Revision 1.6  1997/08/31  02:15:58  eshayne
 * added form handler
 *
 * Revision 1.5  1997/08/20  02:39:56  eshayne
 * added second player's timer and functionality to switch between the two
 * instead of just freezing
 *
 * Revision 1.4  1997/08/20  02:04:57  eshayne
 * adjusted to use clockResDefs for resource IDs
 *
 * Revision 1.3  1997/08/20  01:42:16  eshayne
 * moved TimeCounter to clockGlobals.h
 * moved stuff into helper functions for clarity
 *
 * Revision 1.2  1997/08/17  17:53:16  eshayne
 * added comment header block
 *

    Game Timer
    Copyright (C) 1997-2000  Ethan Shayne

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*******************************************************************************/

#include <Pilot.h>
#include <SysEvtMgr.h>

#include "counter.h"
#include "clockGlobals.h"
#include "GameTimerRsc.h"


/* standard application functions */
static int StartApplication();
static void EventLoop();
static void StopApplication();

/* form event handlers */
Boolean ClockMainFormHandler(EventPtr pEvent);
Boolean ClockStartPreferencesFormHandler(EventPtr pEvent);
Boolean ClockAdjustPreferencesFormHandler(EventPtr pEvent);
Boolean ClockMiscPreferencesFormHandler(EventPtr pEvent);

/* helper functions */
static void Reset();
static Boolean GetStartupPreferencesFromForm(FormPtr pForm);
static Boolean GetAdjustPreferencesFromForm(FormPtr pForm);
static Boolean GetMiscPreferencesFromForm(FormPtr pForm);
static void PrefillStartupPreferencesData(FormPtr pForm);
static void PrefillAdjustPreferencesData(FormPtr pForm);
static void PrefillMiscPreferencesData(FormPtr pForm);
static void SetFieldText(FieldPtr pField, CharPtr pszText);
static Boolean ValidateMiscPreferences(CharPtr pszAccuracy,
						   			   short sPossiblePauseEnabled,
				   					   short sPossibleHardwareButtonsEnabled,
				   					   short sPossibleAutoOffDisabled,
				   					   unsigned long* pulAccuracy,
				   					   int* piPauseEnabled,
				   					   int* piHardwareButtonsEnabled,
				   					   int* piDisableAutoOff);
static Boolean ValidateStartupPreferences(CharPtr pszInitMinutes,
										  CharPtr pszInitSeconds,
										  CharPtr pszHandicap1Minutes,
										  CharPtr pszHandicap1Seconds,
										  CharPtr pszHandicap2Minutes,
										  CharPtr pszHandicap2Seconds,
				   						  unsigned long* pulInitMinutes,
									      unsigned long* pulInitSeconds,
									      unsigned long* pulHandicap1Minutes,
									      unsigned long* pulHandicap1Seconds,
									      unsigned long* pulHandicap2Minutes,
									      unsigned long* pulHandicap2Seconds);
static Boolean ValidateAdjustmentPreferences(CharPtr pszFischer,
											 short sFischerAtEnd,
											 short sResetAtEnd,
											 unsigned long* pulFischer,
											 int* piFischerAtEnd,
											 int* piResetAtEnd);
static void GetPreferences();
static void SavePreferences();
static void SetInitialValues();
static void DrawInitialFields(FormPtr pForm, const Boolean boolInitial);
static void ToggleSwitchButtons(const enum SwitchButtonToggles eToggleType);
static void ResetDefaultPrefs();
static void Reset14DefaultPrefs();
static void ResetTimers();
static void ResetTimer1();
static void ResetTimer2();
static void NormalizeTime(int* piMinutes,
						  int* piSeconds);


/* PilotMain */
DWord PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
  Word error;
    
  if (cmd == sysAppLaunchCmdNormalLaunch) {
    error = StartApplication();
    if (error) return error;

    EventLoop();

    StopApplication();
  }

  return 0;
}


static int StartApplication()
{
  ResetDefaultPrefs();
  FrmGotoForm(ES_CLOCK_Form);
  return 0;
}


/* Event Loop */
static void EventLoop()
{
  EventType event;
  Word err;
  int formId;
  FormPtr form;

  do {
    /* get the next event in the queue */
    EvtGetEvent(&event, prefs.ulWait);

    /* let the system have first shot at it */
    /* ... but only if it's not an event indicating that
       hardware button 1 or 4 has been pressed, because we'll
       handle that (interpreted as a "switch" button press) */
    /* ... but only intercept it before the system gets to it
       if our preferences say that hardware buttons are enabled */
    if (!prefs.iHardwareButtonsEnabled ||
	!((event.eType == keyDownEvent)
	  && ((event.data.keyDown.chr == hard1Chr)
	      || (event.data.keyDown.chr == hard4Chr))))
      {
	if (SysHandleEvent(&event)) continue;
      }
    if (MenuHandleEvent(NULL, &event, &err)) continue;

    switch (event.eType)
      {
	/* if we're loading a new form, load it and set the event handler */
      case frmLoadEvent:
	formId = event.data.frmLoad.formID;
	form = FrmInitForm(formId);
	FrmSetActiveForm(form);
	
	switch (formId)
	  {
	    /* if it's the main form, use the main form handler */
	  case ES_CLOCK_Form:
	    FrmSetEventHandler(form, ClockMainFormHandler);
	    break;

	  case ES_CLOCK_ADJUST_PREF_Form:
	    FrmSetEventHandler(form, ClockAdjustPreferencesFormHandler);
	    break;

	  case ES_CLOCK_MISC_PREF_Form:
	    FrmSetEventHandler(form, ClockMiscPreferencesFormHandler);
	    break;

	  case ES_CLOCK_START_PREF_Form:
	    FrmSetEventHandler(form, ClockStartPreferencesFormHandler);
	    break;
	  } 

	break;
      }


    /* if we've got everything set up (the form's been opened and drawn)
       then every loop we should decrement whichever timer is currently active */
    /* ... but only if we're on the main form */
    /* ... and only if we've actually started (i.e. someone's pressed
       the button) */
    if (iInitialized &&
	(FrmGetActiveFormID() == ES_CLOCK_Form) &&
	(iFirstPlayerStarted || iSecondPlayerStarted) &&
	!iPaused)
      {
	if (iCurrentPlayer == 1) {
	  if (!UpdateTimer(&ulNextCountTick,
			   		   &counter1,
				   	   pMinutesField1,
			   		   pSecondsField1))
	    {
	      FrmCustomAlert(ES_CLOCK_GAMEOVER_Alert, "1", "", "");
	      Reset();
	    }
	}
	else if (iCurrentPlayer == 2) {
	  if (!UpdateTimer(&ulNextCountTick,
			   		   &counter2,
			   		   pMinutesField2,
			   		   pSecondsField2))
	    {
	      FrmCustomAlert(ES_CLOCK_GAMEOVER_Alert, "2", "", "");
	      Reset();
	    }
	}
      }

    /* if nothing else handled the event, dispatch it to the current event
       handler */
    FrmDispatchEvent(&event);

    /*keep going until the event is a "stop" event (application terminated) */
  } while (event.eType != appStopEvent);
}


static void StopApplication()
{
}


/* event handler for the main form */
Boolean ClockMainFormHandler(EventPtr pEvent)
{
	Boolean boolHandled = false;
	FormPtr form = FrmGetActiveForm();
	Word wObjIndex;
	ControlPtr pButton;
	unsigned long ulOldNextTick = 0;

	switch (pEvent->eType)
    {
    	// if we're just opening the form for the first time...
    	case frmOpenEvent:
      		/* initalize the initial values of the counters, labels, etc. */
      		SetInitialValues();

	    	/* draw the form */
      		FrmDrawForm(form);

      		/* set up some global field pointers for later use, just so
	 		we don't need to keep finding them every time */
      		wObjIndex = FrmGetObjectIndex(form, ES_CLOCK_PLAYER1_MINUTES_Field);
      		pMinutesField1 = FrmGetObjectPtr(form, wObjIndex);
      		wObjIndex = FrmGetObjectIndex(form, ES_CLOCK_PLAYER1_SECONDS_Field);
      		pSecondsField1 = FrmGetObjectPtr(form, wObjIndex);
      		wObjIndex = FrmGetObjectIndex(form, ES_CLOCK_PLAYER2_MINUTES_Field);
      		pMinutesField2 = FrmGetObjectPtr(form, wObjIndex);
      		wObjIndex = FrmGetObjectIndex(form, ES_CLOCK_PLAYER2_SECONDS_Field);
      		pSecondsField2 = FrmGetObjectPtr(form, wObjIndex);

      		/* draw in some of the initial actual field data */
      		DrawInitialFields(form, true);

      		iInitialized = 1;
      		boolHandled = true;
      		break;
      
		/* if the event was a press of a button on the form... */
    	case ctlSelectEvent:
      		/* if the button was our "switch" button... */
      		/* we only care if:
      			- it's player 1's turn and button 1 was pressed;
      			- it's player 2's turn and button 2 was pressed;
      			- or the start button was pressed. 
      		if the switch or start button was pressed and these conditions don't hold,
      		ignore this event */
      		if (((pEvent->data.ctlEnter.controlID == ES_CLOCK_SWITCH1_Button) && (iCurrentPlayer == 1)) ||
	  			((pEvent->data.ctlEnter.controlID == ES_CLOCK_SWITCH2_Button) && (iCurrentPlayer == 2)) ||
			  	(pEvent->data.ctlEnter.controlID == ES_CLOCK_START_Button))
			{
				// is this a switch or a start?
				Boolean bSwitch = false;
				if ((pEvent->data.ctlEnter.controlID == ES_CLOCK_SWITCH1_Button) ||
					(pEvent->data.ctlEnter.controlID == ES_CLOCK_SWITCH2_Button))
				{
					bSwitch = true;
				}
			
	  			/* only pay attention if we're not paused */
				if (!iPaused) {
					Boolean bWasTimer1Changed = false;
					Boolean bWasTimer2Changed = false;
					
					if (prefs.iResetTimersEachTurn) {
						// if we're supposed to reset the timers each turn, then
						// reset the timer for the player being switched to.
						if (iCurrentPlayer == 1) {
							ResetTimer2();
							bWasTimer2Changed = true;
						}
						else {
							ResetTimer1();
							bWasTimer1Changed = true;
						}

						// set up the next time we need to change the counters
			    		ulNextCountTick = TimGetTicks() + sysTicksPerSecond;
					}
					else {
						// if we're not resetting the timers each turn, then
						// calculate the new timer value(s)

						Boolean bHasBeenSwitchedTo = false;
						Boolean bHasBeenSwitchedFrom = false;
						// have we switched to the player we're about to switch to
						// before?
						if (((iCurrentPlayer == 1) && iSecondPlayerStarted) ||
							((iCurrentPlayer == 2) && iFirstPlayerStarted))
						{
							bHasBeenSwitchedTo = true;
						}
						
						if (((iCurrentPlayer == 1) && iFirstPlayerStarted) ||
							((iCurrentPlayer == 2) && iSecondPlayerStarted))
						{
							bHasBeenSwitchedFrom = true;
						}
						
		    			ulOldNextTick = ulNextCountTick;
	  
			  			// set the next time we're going to want to adjust the counter (ulNextCountTick)
			    		/* if the player we're about to switch to has been switched
			       		away from yet, set the next decrement marker based on how
			       		much time was left last time we switched away from this player.
			       		otherwise just use sysTicksPerSecond. */
			    		if (bHasBeenSwitchedTo)
			      		{
							ulNextCountTick = TimGetTicks() + ulLastLeftoverTicks;
			      		}
			    		else
				    		ulNextCountTick = TimGetTicks() + sysTicksPerSecond;	
		    
				    	/* if we're switching (and not just starting) - remember how
			       		much time was leftover before the next counter decrement
			       		so we can take that into account when we switch back */
			       		// basically, we don't want to lose fractions of a second
			    		if (bHasBeenSwitchedFrom)
			    			ulLastLeftoverTicks = ulOldNextTick - TimGetTicks();

						// handle Fischer time if necessary
						if (prefs.ulFischer) {
							// depending on the preferences, we change a different counter
							// at the end of this player's turn. if the preferences say that
							// we should implement Fischer time at the end of a turn, then
							// increment the current player's counter; otherwise increment
							// the other player's counter at this time.
							struct TimeCounter* pFischerCounter = NULL;
							if ((prefs.iFischerAtEnd && iCurrentPlayer == 1) ||
								(!prefs.iFischerAtEnd && iCurrentPlayer == 2))
							{
								pFischerCounter = &counter1;
							}
							else
								pFischerCounter = &counter2;
								
							// only add Fischer time to a clock which has already run. In
							// other words, never add Fischer time to a clock which is about
							// to begin for the first time in a game.
							// Note that if we're adding Fischer time at the end of that player's
							// turn (preferences), then that player *has* run - so it's ok. BUT,
							// in this case, note that if this is a "start" call, then that does
							// NOT count.
							if (((pFischerCounter == &counter1) && !iFirstPlayerStarted) ||
								((pFischerCounter == &counter2) && !iSecondPlayerStarted))
							{
								if (!(bSwitch && prefs.iFischerAtEnd))
									pFischerCounter = NULL;
							}

							if (pFischerCounter)
								IncrementCounter(pFischerCounter, prefs.ulFischer);
							
							if (pFischerCounter == &counter1)
								bWasTimer1Changed = true;
								
							if (pFischerCounter == &counter2)
								bWasTimer2Changed = true;

						}
					} // if (!prefs.iResetTimersEachTurn)


					// change the current player number
					if (iCurrentPlayer == 1) {
						iCurrentPlayer = 2;

		      			/* if this was this player's first move, next one won't be */
		      			// if we "switched" to player #2 by hitting the start button, then
		      			// that does NOT mean that player #1 has "started."
		      			if (!iFirstPlayerStarted && bSwitch)
							iFirstPlayerStarted = 1;
							
						// however, if we did get here by hitting the start button, then
						// that means that player #2 has in fact "started."
						if (!bSwitch)
							iSecondPlayerStarted = 1;
					}
					else {
						iCurrentPlayer = 1;

		      			/* if this was this player's first move, next one won't be */
		      			// if we "switched" to player #1 by hitting the start button, then
		      			// that does NOT mean that player #2 has "started."
		      			if (!iSecondPlayerStarted && bSwitch)
							iSecondPlayerStarted = 1;
							
						// however, if we did get here by hitting the start button, then
						// that means that player #1 has in fact "started."
						if (!bSwitch)
							iFirstPlayerStarted = 1;
					}
					
					// whether we're resetting timers each turn or not, redraw
					// the timer(s) we just changed
					if (bWasTimer1Changed) {
						DrawTimer(&counter1,
					  			  pMinutesField1,
					  			  pSecondsField1);
					}
					
					if (bWasTimer2Changed) {
						DrawTimer(&counter2,
					  			  pMinutesField2,
					  			  pSecondsField2);
					}

					/* if this was the first button press, switch the button's
	       		    label to "Switch" */
	    		   	/* ... but only if hardware buttons are not enabled */
	    			if (!prefs.iHardwareButtonsEnabled &&
						(!iFirstPlayerStarted || !iSecondPlayerStarted))
	      			{
						ToggleSwitchButtons(SWITCH);
	      			}
				} // if (!iPaused)
			} // if we cared about this event
      
			// if the Pause button was pressed...
			// may apply
			else if (pEvent->data.ctlEnter.controlID == ES_CLOCK_PAUSE_Button) {
				/* pause the countdown if the pause button was pressed */
				/* if we haven't started yet, ignore the pause request */
				if (iFirstPlayerStarted || iSecondPlayerStarted) {
	  				/* see if we're pausing or un-pausing */
	  				if (!iPaused) {
	    				/* set the flag so the next event loop won't continue counting */
	    				iPaused = 1;
	    				/* remember how far we were from a counter decrement... */
	    				ulLastLeftoverPauseTicks = ulNextCountTick - TimGetTicks();
	  				}
	  				else {
	    				/* if we're un-pausing... */
	    				iPaused = 0;
	    				ulNextCountTick = TimGetTicks() + ulLastLeftoverPauseTicks;
	  				}
				} // if we've started yet
      		} // if the pause button was pressed

			boolHandled = true;
      		break;

		// if a menu selection happened
		case menuEvent:
      		switch (pEvent->data.menu.itemID)
			{
				case MainOptionsAboutGameTimer:
	  				FrmAlert(ES_CLOCK_ABOUT_Alert);
	  				boolHandled = true;
	  				break;

				case MainOptionsStartupSettings:
	  				FrmPopupForm(ES_CLOCK_START_PREF_Form);
	  				boolHandled = true;
	  				break;

				case MainOptionsAdjustmentSettings:
	  				FrmPopupForm(ES_CLOCK_ADJUST_PREF_Form);
	  				boolHandled = true;
	  				break;

				case MainOptionsMiscellaneousSettings:
	  				FrmPopupForm(ES_CLOCK_MISC_PREF_Form);
	  				boolHandled = true;
	  				break;

				case MainOptionsReset:
	  				if (!FrmAlert(ES_CLOCK_CONFIRM_RESET_Alert))
	    				Reset();
	  				boolHandled = true;
	  				break;
			}

    	/* if one of the hardware buttons was pressed... */
    	case keyDownEvent:
      		/* in the somewhat "special" situation where we haven't started
	 		yet, but we are using the hardware buttons, *either* hardware
	 		button should be interpretable as "start" - in other words,
	 		just as if when hardware buttons are disabled, the user had
	 		hit the "start" button. */
      		if ((!iFirstPlayerStarted && !iSecondPlayerStarted) &&
	  			((pEvent->data.keyDown.chr == hard1Chr) || (pEvent->data.keyDown.chr == hard4Chr)))
			{
	  			wObjIndex = FrmGetObjectIndex(form, ES_CLOCK_START_Button);
	  			pButton = FrmGetObjectPtr(form, wObjIndex);
	  			CtlHitControl(pButton);

	  			boolHandled = true;
			}

      		/* otherwise, the hardware buttons should be interpreted as
	 		either player 1's button or player 2's button */
      		else {
				/* if it was the first hardware button (the calendar one),
	   			pretend the user hit switch button #1 */
				if (pEvent->data.keyDown.chr == hard1Chr) {
	  				wObjIndex = FrmGetObjectIndex(form, ES_CLOCK_SWITCH1_Button);
	  				pButton = FrmGetObjectPtr(form, wObjIndex);
	  				CtlHitControl(pButton);
	  
	  				boolHandled = true;
				}
	
				/* if it was the fourth hardware button (the memopad one),
	   			pretend the user hit switch button #2 */
				if (pEvent->data.keyDown.chr == hard4Chr) {
	  				wObjIndex = FrmGetObjectIndex(form, ES_CLOCK_SWITCH2_Button);
	  				pButton = FrmGetObjectPtr(form, wObjIndex);
	  				CtlHitControl(pButton);
	  
	  				boolHandled = true;
				}
      		} // hardware button hit in non-start situation
	} // switch (pEvent->eType)

	// return a boolean indicating if we did anything with this event,
	// or if the system should try to handle it
  	return boolHandled;
}


Boolean ClockMiscPreferencesFormHandler(EventPtr pEvent)
{
  Boolean boolHandled = false;
  FormPtr form = FrmGetActiveForm();

  switch (pEvent->eType)
    {
    case frmOpenEvent:
      /* draw the form */
      PrefillMiscPreferencesData(form);
      FrmDrawForm(form);
      boolHandled = true;
      break;

    case ctlSelectEvent:
      if (pEvent->data.ctlEnter.controlID == ES_CLOCK_MISC_PREF_OK_Button) {
		/* take the values from the form and set the global variables */
		if (GetMiscPreferencesFromForm(form)) {
	  		FrmReturnToForm(ES_CLOCK_Form);
	  		Reset();
		}
		boolHandled = true;
      }
      else if (pEvent->data.ctlEnter.controlID == ES_CLOCK_MISC_PREF_CANCEL_Button) {
		FrmReturnToForm(ES_CLOCK_Form);
		boolHandled = true;
      }

      break;
    }

  return boolHandled;
}


Boolean ClockStartPreferencesFormHandler(EventPtr pEvent)
{
  Boolean boolHandled = false;
  FormPtr form = FrmGetActiveForm();

  switch (pEvent->eType)
    {
    case frmOpenEvent:
      /* draw the form */
      PrefillStartupPreferencesData(form);
      FrmDrawForm(form);
      boolHandled = true;
      break;

    case ctlSelectEvent:
      if (pEvent->data.ctlEnter.controlID == ES_CLOCK_START_PREF_OK_Button) {
		/* take the values from the form and set the global variables */
		if (GetStartupPreferencesFromForm(form)) {
	  		FrmReturnToForm(ES_CLOCK_Form);
	  		Reset();
		}
		boolHandled = true;
      }
      else if (pEvent->data.ctlEnter.controlID == ES_CLOCK_START_PREF_CANCEL_Button) {
		FrmReturnToForm(ES_CLOCK_Form);
		boolHandled = true;
      }

      break;
    }

  return boolHandled;
}


Boolean ClockAdjustPreferencesFormHandler(EventPtr pEvent)
{
  Boolean boolHandled = false;
  FormPtr form = FrmGetActiveForm();

  switch (pEvent->eType)
    {
    case frmOpenEvent:
      /* draw the form */
      PrefillAdjustPreferencesData(form);
      FrmDrawForm(form);
      boolHandled = true;
      break;

    case ctlSelectEvent:
      if (pEvent->data.ctlEnter.controlID == ES_CLOCK_ADJUST_PREF_OK_Button) {
		/* take the values from the form and set the global variables */
		if (GetAdjustPreferencesFromForm(form)) {
	  		FrmReturnToForm(ES_CLOCK_Form);
	  		Reset();
		}
		boolHandled = true;
      }
      else if (pEvent->data.ctlEnter.controlID == ES_CLOCK_ADJUST_PREF_CANCEL_Button) {
		FrmReturnToForm(ES_CLOCK_Form);
		boolHandled = true;
      }

      break;
    }

  return boolHandled;
}


static void Reset()
{
  FormPtr pForm = FrmGetActiveForm();

  SetInitialValues();
  DrawInitialFields(pForm, false);
}


static Boolean GetMiscPreferencesFromForm(FormPtr pForm)
{
  Word wObjIndex;
  FieldPtr pAccuracy;
  ControlPtr pPauseEnabled;
  ControlPtr pHardwareEnabled;
  ControlPtr pAutoOffDisabled;

  CharPtr pszAccuracy = NULL;
  short sTempPauseEnabled = 0;
  short sTempHardwareEnabled = 0;
  short sTempAutoOffDisabled = 0;

  Boolean boolReturn;

  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_MISC_PREF_ACCURACY_Field);
  pAccuracy = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_MISC_PREF_PAUSE_ENABLED_Checkbox);
  pPauseEnabled = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_MISC_PREF_HARDWARE_ENABLED_Checkbox);
  pHardwareEnabled = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_MISC_PREF_AUTOOFF_DISABLED_Checkbox);
  pAutoOffDisabled = FrmGetObjectPtr(pForm, wObjIndex);
  
  pszAccuracy = FldGetTextPtr(pAccuracy);
  sTempPauseEnabled = CtlGetValue(pPauseEnabled);
  sTempHardwareEnabled = CtlGetValue(pHardwareEnabled);
  sTempAutoOffDisabled = CtlGetValue(pAutoOffDisabled);
  
  boolReturn = ValidateMiscPreferences(pszAccuracy,
				   					   sTempPauseEnabled,
				   					   sTempHardwareEnabled,
				  		 			   sTempAutoOffDisabled,
				   					   &prefs.ulWait,
				   					   &prefs.iPauseEnabled,
				   					   &prefs.iHardwareButtonsEnabled,
				   					   &prefs.iAutoOffDisabled);

  if (boolReturn)
    SavePreferences();

  return boolReturn;
}


static Boolean GetAdjustPreferencesFromForm(FormPtr pForm)
{
  Word wObjIndex;
  FieldPtr pFischerSeconds;
  ControlPtr pFischerAtEnd;
  ControlPtr pResetAtEnd;

  CharPtr pszFischerSeconds = NULL;
  short sTempFischerAtEnd = 0;
  short sTempResetAtEnd = 0;

  Boolean boolReturn;

  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_ADJUST_PREF_FISCHER_SECONDS_Field);
  pFischerSeconds = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_ADJUST_PREF_FISCHER_AT_END_Checkbox);
  pFischerAtEnd = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_ADJUST_PREF_RESET_AT_END_Checkbox);
  pResetAtEnd = FrmGetObjectPtr(pForm, wObjIndex);
  
  pszFischerSeconds = FldGetTextPtr(pFischerSeconds);
  sTempFischerAtEnd = CtlGetValue(pFischerAtEnd);
  sTempResetAtEnd = CtlGetValue(pResetAtEnd);
  
  boolReturn = ValidateAdjustmentPreferences(pszFischerSeconds,
									   	 	 sTempFischerAtEnd,
									   	 	 sTempResetAtEnd,
				   						 	 &prefs.ulFischer,
				   						 	 &prefs.iFischerAtEnd,
				   						 	 &prefs.iResetTimersEachTurn);

  if (boolReturn)
    SavePreferences();

  return boolReturn;
}


static Boolean GetStartupPreferencesFromForm(FormPtr pForm)
{
  Word wObjIndex;
  FieldPtr pInitMinutes;
  FieldPtr pInitSeconds;
  FieldPtr pHandicap1Minutes;
  FieldPtr pHandicap1Seconds;
  FieldPtr pHandicap2Minutes;
  FieldPtr pHandicap2Seconds;

  CharPtr pszInitMinutes = NULL;
  CharPtr pszInitSeconds = NULL;
  CharPtr pszHandicap1Minutes = NULL;
  CharPtr pszHandicap1Seconds = NULL;
  CharPtr pszHandicap2Minutes = NULL;
  CharPtr pszHandicap2Seconds = NULL;

  Boolean boolReturn;

  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_INIT_MIN_Field);
  pInitMinutes = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_INIT_SEC_Field);
  pInitSeconds = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_HANDICAP_PLAYER1_MIN_Field);
  pHandicap1Minutes = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_HANDICAP_PLAYER1_SEC_Field);
  pHandicap1Seconds = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_HANDICAP_PLAYER2_MIN_Field);
  pHandicap2Minutes = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_HANDICAP_PLAYER2_SEC_Field);
  pHandicap2Seconds = FrmGetObjectPtr(pForm, wObjIndex);

  pszInitMinutes = FldGetTextPtr(pInitMinutes);
  pszInitSeconds = FldGetTextPtr(pInitSeconds);
  pszHandicap1Minutes = FldGetTextPtr(pHandicap1Minutes);
  pszHandicap1Seconds = FldGetTextPtr(pHandicap1Seconds);
  pszHandicap2Minutes = FldGetTextPtr(pHandicap2Minutes);
  pszHandicap2Seconds = FldGetTextPtr(pHandicap2Seconds);
  
  boolReturn = ValidateStartupPreferences(pszInitMinutes,
				   pszInitSeconds,
				   pszHandicap1Minutes,
				   pszHandicap1Seconds,
				   pszHandicap2Minutes,
				   pszHandicap2Seconds,
				   &prefs.ulStartMinutes,
				   &prefs.ulStartSeconds,
				   &prefs.ulPlayer1HandicapMinutes,
				   &prefs.ulPlayer1HandicapSeconds,
				   &prefs.ulPlayer2HandicapMinutes,
				   &prefs.ulPlayer2HandicapSeconds);

  if (boolReturn)
    SavePreferences();

  return boolReturn;
}


static void PrefillMiscPreferencesData(FormPtr pForm)
{
  static char pszAccuracy[5];

  Word wObjIndex;
  FieldPtr pAccuracy;
  ControlPtr pPauseEnabled;
  ControlPtr pHardwareEnabled;
  ControlPtr pAutoOffDisabled;

  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_MISC_PREF_ACCURACY_Field);
  pAccuracy = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_MISC_PREF_PAUSE_ENABLED_Checkbox);
  pPauseEnabled = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_MISC_PREF_HARDWARE_ENABLED_Checkbox);
  pHardwareEnabled = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_MISC_PREF_AUTOOFF_DISABLED_Checkbox);
  pAutoOffDisabled = FrmGetObjectPtr(pForm, wObjIndex);
  
  StrIToA(pszAccuracy, prefs.ulWait);

  SetFieldText(pAccuracy, pszAccuracy);
  CtlSetValue(pPauseEnabled, prefs.iPauseEnabled);
  CtlSetValue(pHardwareEnabled, prefs.iHardwareButtonsEnabled);
  CtlSetValue(pAutoOffDisabled, prefs.iAutoOffDisabled);
}


static void PrefillAdjustPreferencesData(FormPtr pForm)
{
  static char pszFischer[5];

  Word wObjIndex;
  FieldPtr pFischerSeconds;
  ControlPtr pFischerAtEnd;
  ControlPtr pResetAtEnd;

  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_ADJUST_PREF_FISCHER_SECONDS_Field);
  pFischerSeconds = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_ADJUST_PREF_FISCHER_AT_END_Checkbox);
  pFischerAtEnd = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_ADJUST_PREF_RESET_AT_END_Checkbox);
  pResetAtEnd = FrmGetObjectPtr(pForm, wObjIndex);
  
  StrIToA(pszFischer, prefs.ulFischer);

  SetFieldText(pFischerSeconds, pszFischer);
  CtlSetValue(pFischerAtEnd, prefs.iFischerAtEnd);
  CtlSetValue(pResetAtEnd, prefs.iResetTimersEachTurn);
}


static void PrefillStartupPreferencesData(FormPtr pForm)
{
  static char pszInitMinutes[5];
  static char pszInitSeconds[5];
  static char pszHandicap1Minutes[5];
  static char pszHandicap1Seconds[5];
  static char pszHandicap2Minutes[5];
  static char pszHandicap2Seconds[5];

  Word wObjIndex;
  FieldPtr pInitMinutes;
  FieldPtr pInitSeconds;
  FieldPtr pHandicap1Minutes;
  FieldPtr pHandicap1Seconds;
  FieldPtr pHandicap2Minutes;
  FieldPtr pHandicap2Seconds;

  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_INIT_MIN_Field);
  pInitMinutes = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_INIT_SEC_Field);
  pInitSeconds = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_HANDICAP_PLAYER1_MIN_Field);
  pHandicap1Minutes = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_HANDICAP_PLAYER1_SEC_Field);
  pHandicap1Seconds = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_HANDICAP_PLAYER2_MIN_Field);
  pHandicap2Minutes = FrmGetObjectPtr(pForm, wObjIndex);
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_PREF_HANDICAP_PLAYER2_SEC_Field);
  pHandicap2Seconds = FrmGetObjectPtr(pForm, wObjIndex);
  
  StrIToA(pszInitMinutes, prefs.ulStartMinutes);
  StrIToA(pszInitSeconds, prefs.ulStartSeconds);
  StrIToA(pszHandicap1Minutes, prefs.ulPlayer1HandicapMinutes);
  StrIToA(pszHandicap1Seconds, prefs.ulPlayer1HandicapSeconds);
  StrIToA(pszHandicap2Minutes, prefs.ulPlayer2HandicapMinutes);
  StrIToA(pszHandicap2Seconds, prefs.ulPlayer2HandicapSeconds);

  SetFieldText(pInitMinutes, pszInitMinutes);
  SetFieldText(pInitSeconds, pszInitSeconds);
  SetFieldText(pHandicap1Minutes, pszHandicap1Minutes);
  SetFieldText(pHandicap1Seconds, pszHandicap1Seconds);
  SetFieldText(pHandicap2Minutes, pszHandicap2Minutes);
  SetFieldText(pHandicap2Seconds, pszHandicap2Seconds);
}


/* Function puts the supplied text into the supplied field. The supplied
   field can be an editable field (so FieldSetTextPtr is not permissible).
   Thanks to Ken Hancock for describing how to write this function. */
static void SetFieldText(FieldPtr pField, CharPtr pszNewText)
{
  VoidHand hFieldText;
  CharPtr pszActualText;
  
  /* first, free any existing data */
  FldFreeMemory(pField);
  
  /* allocate a new handle for the new text */
  hFieldText = MemHandleNew(StrLen(pszNewText) + 1);

  /* copy the supplied data into the new handle */
  /* get a pointer to the actual data held by the handle (& lock it) */
  pszActualText = MemHandleLock(hFieldText);
  StrCopy(pszActualText, pszNewText);
  
  /* set the field to use the new handle */
  FldSetTextHandle(pField, (Handle)hFieldText);

  /* unlock the handle */
  MemHandleUnlock(hFieldText);
}


static Boolean ValidateMiscPreferences(CharPtr pszAccuracy,
						   			   short sPossiblePauseEnabled,
				   					   short sPossibleHardwareEnabled,
				   					   short sPossibleAutoOffDisabled,
				   					   unsigned long* pulAccuracy,
				   					   int* piPauseEnabled,
				   					   int* piHardwareEnabled,
				   					   int* piAutoOffDisabled)
{
  int iTempAccuracy;

  if (!pszAccuracy) {
    FrmCustomAlert(ES_CLOCK_PREFS_INVALID_NULL_Alert, "Accuracy", "" , "");
    return false;
  }

  iTempAccuracy = StrAToI(pszAccuracy);

  if ((iTempAccuracy < 1) || (iTempAccuracy > 100)) {
    FrmCustomAlert(ES_CLOCK_PREFS_INVALID_VALUE_Alert, "Accuracy", "1" , "100");
    return false;
  }

  *pulAccuracy = iTempAccuracy;
  *piPauseEnabled = sPossiblePauseEnabled;
  *piHardwareEnabled = sPossibleHardwareEnabled;
  *piAutoOffDisabled = sPossibleAutoOffDisabled;

  return true;
}


static Boolean ValidateAdjustmentPreferences(CharPtr pszFischer,
				   							 short sFischerAtEnd,
				   							 short sResetAtEnd,
				   							 unsigned long* pulFischer,
				   							 int* piFischerAtEnd,
				   							 int* piResetAtEnd)
{
  int iTempFischer = 0;
  
  if (pszFischer)
	iTempFischer = StrAToI(pszFischer);

  if (iTempFischer < 0) {
    FrmCustomAlert(ES_CLOCK_PREFS_INVALID_VALUE_Alert, "Fischer Time", "0" , "");
    return false;
  }

  *pulFischer = iTempFischer;
  *piFischerAtEnd = sFischerAtEnd;
  *piResetAtEnd = sResetAtEnd;

  return true;
}


static Boolean ValidateStartupPreferences(CharPtr pszInitMinutes,
				   						  CharPtr pszInitSeconds,
										  CharPtr pszHandicap1Minutes,
										  CharPtr pszHandicap1Seconds,
										  CharPtr pszHandicap2Minutes,
										  CharPtr pszHandicap2Seconds,
				   						  unsigned long* pulInitMinutes,
									      unsigned long* pulInitSeconds,
									      unsigned long* pulHandicap1Minutes,
									      unsigned long* pulHandicap1Seconds,
									      unsigned long* pulHandicap2Minutes,
									      unsigned long* pulHandicap2Seconds)
{
  int iTempInitMinutes;
  int iTempInitSeconds;
  int iTempHandicap1Minutes = 0;
  int iTempHandicap1Seconds = 0;
  int iTempHandicap2Minutes = 0;
  int iTempHandicap2Seconds = 0;
  int iTempTotalInitSeconds = 0;
  int iTempTotalHandicap1Seconds = 0;
  int iTempTotalHandicap2Seconds = 0;
  int iTempAdjusted1Seconds = 0;
  int iTempAdjusted2Seconds = 0;

  if (!pszInitMinutes || !pszInitSeconds) {
    if (!pszInitMinutes)
      FrmCustomAlert(ES_CLOCK_PREFS_INVALID_NULL_Alert, "Initial Minutes", "" , "");
    if (!pszInitSeconds)
      FrmCustomAlert(ES_CLOCK_PREFS_INVALID_NULL_Alert, "Initial Seconds", "" , "");
    return false;
  }

  iTempInitMinutes = StrAToI(pszInitMinutes);
  iTempInitSeconds = StrAToI(pszInitSeconds);
  if (pszHandicap1Minutes)
  	iTempHandicap1Minutes = StrAToI(pszHandicap1Minutes);
  if (pszHandicap1Seconds)
  	iTempHandicap1Seconds = StrAToI(pszHandicap1Seconds);
  if (pszHandicap2Minutes)
  	iTempHandicap2Minutes = StrAToI(pszHandicap2Minutes);
  if (pszHandicap2Seconds)
  	iTempHandicap2Seconds = StrAToI(pszHandicap2Seconds);

  if ((iTempInitMinutes < 0) || (iTempInitMinutes > 59)) {
    FrmCustomAlert(ES_CLOCK_PREFS_INVALID_VALUE_Alert, "Initial Minutes", "0" , "59");
    return false;
  }

  if ((iTempInitSeconds < 0) || (iTempInitSeconds > 59)) {
    FrmCustomAlert(ES_CLOCK_PREFS_INVALID_VALUE_Alert, "Initial Seconds", "0" , "59");
    return false;
  }

  if ((iTempHandicap1Minutes < 0) || (iTempHandicap1Minutes > 59)) {
    FrmCustomAlert(ES_CLOCK_PREFS_INVALID_VALUE_Alert, "Player 1 Handicap Minutes", "0" , "59");
    return false;
  }

  if ((iTempHandicap1Seconds < 0) || (iTempHandicap1Seconds > 59)) {
    FrmCustomAlert(ES_CLOCK_PREFS_INVALID_VALUE_Alert, "Player 1 Handicap Seconds", "0" , "59");
    return false;
  }

  if ((iTempHandicap2Minutes < 0) || (iTempHandicap2Minutes > 59)) {
    FrmCustomAlert(ES_CLOCK_PREFS_INVALID_VALUE_Alert, "Player 2 Handicap Minutes", "0" , "59");
    return false;
  }

  if ((iTempHandicap2Seconds < 0) || (iTempHandicap2Seconds > 59)) {
    FrmCustomAlert(ES_CLOCK_PREFS_INVALID_VALUE_Alert, "Player 2 Handicap Seconds", "0" , "59");
    return false;
  }
  
  // make sure we don't have any negative values...
  iTempTotalInitSeconds = (iTempInitMinutes * 60) + iTempInitSeconds;
  iTempTotalHandicap1Seconds = (iTempHandicap1Minutes * 60) + iTempHandicap1Seconds;
  iTempTotalHandicap2Seconds = (iTempHandicap2Minutes * 60) + iTempHandicap2Seconds;
  iTempAdjusted1Seconds = iTempTotalInitSeconds - iTempTotalHandicap1Seconds;
  iTempAdjusted2Seconds = iTempTotalInitSeconds - iTempTotalHandicap2Seconds;
  if (iTempAdjusted1Seconds < 0) {
  	FrmCustomAlert(ES_CLOCK_PREFS_INVALID_NEG_Alert, "1", "", "");
  	return false;
  }
  if (iTempAdjusted2Seconds < 0) {
  	FrmCustomAlert(ES_CLOCK_PREFS_INVALID_NEG_Alert, "2", "", "");
  	return false;
  }

  *pulInitMinutes = iTempInitMinutes;
  *pulInitSeconds = iTempInitSeconds;
  *pulHandicap1Minutes = iTempHandicap1Minutes;
  *pulHandicap1Seconds = iTempHandicap1Seconds;
  *pulHandicap2Minutes = iTempHandicap2Minutes;
  *pulHandicap2Seconds = iTempHandicap2Seconds;

  return true;
}


static void ToggleSwitchButtons(const enum SwitchButtonToggles eToggleType)
{
  Word wObjIndex;
  ControlPtr pButton;
  FormPtr pForm = FrmGetActiveForm();

  switch (eToggleType)
    {
    /* if eToggleType is START, then we want to erase the dual
       "switch" buttons and draw the single "start" button. */
    case START:
      /* erase the "switch" buttons */
      wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_SWITCH1_Button);
      pButton = FrmGetObjectPtr(pForm, wObjIndex);
      CtlHideControl(pButton);
      
      wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_SWITCH2_Button);
      pButton = FrmGetObjectPtr(pForm, wObjIndex);
      CtlHideControl(pButton);
      
      /* draw the "start" button */
      wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_Button);
      pButton = FrmGetObjectPtr(pForm, wObjIndex);
      CtlShowControl(pButton);
      
      break;

    /* if eToggleType is SWITCH, erase the "start" button and
       draw the "switch" buttons */
    case SWITCH:
      /* erase the "start" button */
      wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_Button);
      pButton = FrmGetObjectPtr(pForm, wObjIndex);
      CtlHideControl(pButton);
      
      /* draw the "switch" buttons */
      wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_SWITCH1_Button);
      pButton = FrmGetObjectPtr(pForm, wObjIndex);
      CtlShowControl(pButton);
      
      wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_SWITCH2_Button);
      pButton = FrmGetObjectPtr(pForm, wObjIndex);
      CtlShowControl(pButton);

      break;

    /* if eToggleType is HIDDEN, hide all the buttons */
    case HIDDEN:
      wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_SWITCH1_Button);
      pButton = FrmGetObjectPtr(pForm, wObjIndex);
      CtlHideControl(pButton);
      
      wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_SWITCH2_Button);
      pButton = FrmGetObjectPtr(pForm, wObjIndex);
      CtlHideControl(pButton);
      
      /* draw the "start" button */
      wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_START_Button);
      pButton = FrmGetObjectPtr(pForm, wObjIndex);
      CtlHideControl(pButton);
      
      break;      
    }
}


static void Reset14DefaultPrefs()
{
	prefs.ulPlayer1HandicapMinutes = 0;
	prefs.ulPlayer1HandicapSeconds = 0;
	prefs.ulPlayer2HandicapMinutes = 0;
	prefs.ulPlayer2HandicapSeconds = 0;
	prefs.iFischerAtEnd = 0;
	prefs.iResetTimersEachTurn = 0;
}


static void ResetDefaultPrefs()
{
	// resets all preferences to their default values
	prefs.ulStartMinutes = 5;
	prefs.ulStartSeconds = 0;
	prefs.ulFischer = 0;
	prefs.ulWait = 10;
	prefs.iPauseEnabled = 0;
	prefs.iHardwareButtonsEnabled = 0;
	prefs.iAutoOffDisabled = 0;
	Reset14DefaultPrefs();
}


static void GetPreferences()
{
  Word wPrefSize = 0;
  SWord swPrefVersion = 0;
  /* if function returns false, prefs will be unaffected and defaults will
     remain in effect */
  /* note - CREATORID is a macro defined in the Makefile to match the creator
     id with which this app was compiled */
  /* make sure that if the get fails, we have some initial values... */
  ResetDefaultPrefs();
  wPrefSize = sizeof(struct ClockPreferences);
  swPrefVersion = PrefGetAppPreferences(CREATORID, 1, &prefs, &wPrefSize, true);
  switch (swPrefVersion)
  {
  	// if we're dealing with an older version, set the new value defaults and then
  	// re-save
  	case 1:
		Reset14DefaultPrefs();
  		SavePreferences();
  		break;
  		
  	case 14:
  		break;
  		
	// if somehow we wound up with an unknown version, just reset the prefs to
	// the defaults and then save
  	default:
  		ResetDefaultPrefs();
  		SavePreferences();
  }
}


static void SavePreferences()
{
  /* note - CREATORID is a macro defined in the Makefile to match the creator
     id with which this app was compiled */
  Word wPrefSize = sizeof(struct ClockPreferences);
  PrefSetAppPreferences(CREATORID, 1, 14, &prefs, wPrefSize, true);
}


static void ResetTimer1()
{
	counter1.miMinutes = prefs.ulStartMinutes - prefs.ulPlayer1HandicapMinutes;
	counter1.miSeconds = prefs.ulStartSeconds - prefs.ulPlayer1HandicapSeconds;
	NormalizeTime(&counter1.miMinutes, &counter1.miSeconds);
	ulLastLeftoverTicks = 0;
}


static void ResetTimer2()
{
	counter2.miMinutes = prefs.ulStartMinutes - prefs.ulPlayer2HandicapMinutes;
	counter2.miSeconds = prefs.ulStartSeconds - prefs.ulPlayer2HandicapSeconds;
	NormalizeTime(&counter2.miMinutes, &counter2.miSeconds);
	ulLastLeftoverTicks = 0;
}


static void ResetTimers()
{
	// just reset the timers, nothing else
	ResetTimer1();
	ResetTimer2();
}


static void SetInitialValues()
{
  /* set the preferences based on the values stored in the
     Preferences database */
  GetPreferences();

  // reset the timers
  ResetTimers();
  
  /* reset the flags indicating that we haven't started yet */  
  iFirstPlayerStarted = 0;
  iSecondPlayerStarted = 0;
  ulNextCountTick = 0;
  
  // gotta start somewhere... this allows the "start" button to act
  // very similarly to a "switch" button
  iCurrentPlayer = 2;
}


static void DrawInitialFields(FormPtr pForm, const Boolean boolInitial)
{
  Word wObjIndex;
  ControlPtr pPauseButton;

  /* enable or disable the pause button as per preferences */
  wObjIndex = FrmGetObjectIndex(pForm, ES_CLOCK_PAUSE_Button);
  pPauseButton = FrmGetObjectPtr(pForm, wObjIndex);
  if (prefs.iPauseEnabled)
    CtlShowControl(pPauseButton);
  else
    /* if this is the first time the form is being drawn, there's
       no need to hide the pause button because it starts out nonusable */
    if (!boolInitial)
      CtlHideControl(pPauseButton);
  
  /* draw in the initial counter values */
  DrawTimer(&counter1,
	    pMinutesField1,
	    pSecondsField1);
  DrawTimer(&counter2,
	    pMinutesField2,
	    pSecondsField2);

  /* draw or hide the appropriate "switch" buttons */
  /* if the preferences indicate that the hardware buttons are
     enabled, hide all the "switch" buttons */
  if (prefs.iHardwareButtonsEnabled)
    ToggleSwitchButtons(HIDDEN);
  /* otherwise, set the "switch" buttons to show just the "start"
     button */
  else
    ToggleSwitchButtons(START);
}


static void NormalizeTime(int* piMinutes,
						  int* piSeconds)
{
	if (!piMinutes || !piSeconds)
		return;
		
	if (*piSeconds < 0) {
		(*piMinutes)--;
		(*piSeconds) += 60;
		NormalizeTime(piMinutes, piSeconds);
	}
	
	if (*piSeconds > 59) {
		(*piMinutes)++;
		(*piSeconds) -= 60;
		NormalizeTime(piMinutes, piSeconds);
	}
}
