/******************************************************************************
   File:
      CLOCKGLOBALS.H

   Purpose:
      Global variable declarations

   $Revision: 1.3 $
   $Header: D:\devel\pilot\CodeWarrior\GameTimer\Src/clockGlobals.h,v 1.3 2001/01/22 01:50:12 eshayne Exp $
   $Log: clockGlobals.h,v $
 * Revision 1.3  2001/01/22  01:50:12  eshayne
 * added new preferences: handicap, fischer adjustment @ end of turn and reset after each turn
 *
 * Revision 1.2  2000/09/10  23:03:51  eshayne
 * renamed to Game Timer
 * default Fischer Time to 0 (disabled)
 * option to disable auto-off
 *
 * Revision 1.1  2000/09/09  23:25:33  eshayne
 * Initial revision
 *
 * Revision 1.16  1997/09/13  22:49:21  eshayne
 * hide software buttons if hardware buttons are enabled
 *
 * Revision 1.15  1997/09/13  22:24:05  eshayne
 * added preferences option to enable or disable hardware buttons
 *
 * Revision 1.14  1997/09/07  19:11:25  eshayne
 * added GNU Public License to source files
 *
 * Revision 1.13  1997/09/07  18:33:40  eshayne
 * replaced global preference vars with one instance of preference struct
 *
 * Revision 1.12  1997/09/07  18:02:26  eshayne
 * made preferences persistent
 *
 * Revision 1.11  1997/09/01  23:41:59  eshayne
 * added optional pause button
 *
 * Revision 1.10  1997/09/01  20:58:08  eshayne
 * made switching routine more accurate (I think) by capturing amount
 * of time left before next decrement before switching over - so next
 * time we can only wait that long before decrementing again. I'm not
 * convinced it works, but it's pretty much impossible to test...
 *
 * Revision 1.9  1997/09/01  19:34:02  eshayne
 * made button switch label between "Start" and "Switch," appropriately
 *
 * Revision 1.8  1997/09/01  17:24:36  eshayne
 * removed "Fischer Time?" checkbox
 * made preferences actually count if user hits "OK"
 *
 * Revision 1.7  1997/09/01  16:31:20  eshayne
 * added Reset option to menu
 *
 * Revision 1.6  1997/09/01  16:08:30  eshayne
 * added Fischer time
 *
 * Revision 1.5  1997/09/01  14:43:52  eshayne
 * moved all TimeCounter-related stuff to counter.h/.c
 *
 * Revision 1.4  1997/09/01  14:31:12  eshayne
 * added headers to make this file standalone
 *
 * Revision 1.3  1997/09/01  14:26:45  eshayne
 * fixed comment header block
 *
 * Revision 1.2  1997/08/31  02:25:51  eshayne
 * extern-ized globals
 *
 * Revision 1.1  1997/08/20  01:44:16  eshayne
 * Initial revision
 *

    Game Timer
    Copyright (C) 1997-2000  Ethan Shayne

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef ES_CLOCKGLOBALS_H
#define ES_CLOCKGLOBALS_H

#include "counter.h"

#define CREATORID 'CTim'


/* structs */
struct ClockPreferences {
  unsigned long ulStartMinutes;
  unsigned long ulStartSeconds;
  unsigned long ulFischer;
  unsigned long ulWait;
  int iPauseEnabled;
  int iHardwareButtonsEnabled;
  int iAutoOffDisabled;
  unsigned long ulPlayer1HandicapMinutes;
  unsigned long ulPlayer1HandicapSeconds;
  unsigned long ulPlayer2HandicapMinutes;
  unsigned long ulPlayer2HandicapSeconds;
  int iFischerAtEnd;
  int iResetTimersEachTurn;
};


/* global enums */
enum SwitchButtonToggles {START, SWITCH, HIDDEN};


/* fields on forms */
/* main form */
extern FieldPtr pMinutesField1;
extern FieldPtr pSecondsField1;
extern FieldPtr pMinutesField2;
extern FieldPtr pSecondsField2;

extern unsigned long ulNextCountTick;
extern unsigned long ulLastLeftoverTicks;
extern unsigned long ulLastLeftoverPauseTicks;
extern int iCurrentPlayer;
extern struct TimeCounter counter1;
extern struct TimeCounter counter2;
extern int iInitialized;
extern int iFirstPlayerStarted;
extern int iSecondPlayerStarted;
extern int iPaused;

/* preferences-based values */
extern struct ClockPreferences prefs;


#endif
